<?php
namespace Ep\Gath\Plugin;

use WP_Post;

/**
 * Zamijenjuje cijene proizvoda onima iz Gatha.
 *
 * @package Ep\Gath\Plugin
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class Prices extends AbstractGathPlugin
{
    /** @var array */
    protected $resultsById = [];
    /** @var array */
    protected $pricesById = [];
    /** @var array */
    protected $queuedIds = [];
    /** @var array */
    protected $ignoredIds = [];

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('get_post_metadata', [$this, 'get_post_metadata'], 10, 3);
        add_filter('the_posts', [$this, 'the_posts'], 10);
        add_filter('ep_gath_prices', [$this, 'ep_gath_prices']);
    }

    /**
     * @return $this
     */
    public function ep_gath_prices()
    {
        return $this;
    }

    /**
     * Ako je netko dohvatio proizvode iz Gatha, zapamtiti ID-eve jer ćemo
     * trebat vjerovatno dohvatiti cijene nekad.
     *
     * @param WP_Post[] $posts
     *
     * @return mixed
     */
    public function the_posts($posts)
    {
        if (is_array($posts)) {
            foreach ($posts as $post) {
                if ($post->post_type !== 'product') {
                    continue;
                }

                $this->queuedIds[] = $post->ID;
            }
        }

        return $posts;
    }

    public function get_post_metadata($null, $object_id, $meta_key)
    {
        if (!in_array($meta_key, [
            '_net_price',
            '_price',
            '_price_hidden',
            '_rebate_percentage',
            '_regular_price',
            '_regular_price_no_tax',
            '_sale_percentage',
            '_sale_price',
        ]) || in_array($object_id, $this->ignoredIds)) {
            return $null;
        }

        if (!isset($this->resultsById[$object_id])) {
            $this->queuedIds[] = $object_id;
            $this->load();
        }

        if (!isset($this->resultsById[$object_id])) {
            return $null;
        }

        $metadata = $this->getMetadata($object_id);

        if (isset($metadata[$meta_key])) {
            return $metadata[$meta_key];
        }

        $result = $this->resultsById[$object_id];

        switch ($meta_key) {
            case '_price_hidden':
                return $result['HidePrice'] ?? false;
        }

        return '';
    }

    /**
     * @param int $id
     *
     * @return array
     */
    protected function getMetadata(int $id): array
    {
        return $this->gath->isWholesale()
            ? $this->getMetadataWholesale($id)
            : $this->getMetadataRetail($id);
    }

    /**
     * @param int $id
     *
     * @return array
     */
    protected function getMetadataRetail(int $id): array
    {
        if (!isset($this->pricesById[$id])) {
            $result = $this->resultsById[$id];
            $discount = null;

            foreach ($result['Discount'] ?? [] as $disc) {
                if (null === $discount || $disc['NewRetail'] < $discount['NewRetail']) {
                    $discount = $disc;
                }
            }

            $this->pricesById[$id] = [
                '_price' => $discount ? $discount['NewRetail'] : $result['PriceRetail'],
                '_regular_price' => $discount['OriginalRetail'] ?? $result['PriceRetail'],
                '_regular_price_no_tax' => $discount['OriginalWholesale'] ?? $result['PriceWholesale'],
                '_sale_percentage' => $discount['Rate'] ?? '',
                '_sale_price' => $discount['NewRetail'] ?? '',
            ];
        }

        return $this->pricesById[$id];
    }

    /**
     * @param int $id
     *
     * @return array
     */
    protected function getMetadataWholesale(int $id): array
    {
        if (!isset($this->pricesById[$id])) {
            $result = $this->resultsById[$id];
            $rebate = $result['RabPolicy'][0] ?? false;
            $discount = null;

            // Ako imamo neto cijenu koristimo nju umjesto nekih drugih
            if ($rebate && $rebate['PriceWholesaleNetto'] > 0) {
                $rebate['PriceWholesale'] = $rebate['PriceWholesaleNetto'];
                $rebate['Rate'] = 0;
                $rebate['CalculatedNetto'] = $rebate['PriceWholesaleNetto'];
            }

            // tražimo najpovoljniju akciju ako postoji
            foreach ($result['Discount'] ?? [] as $disc) {
                if (null === $discount || $disc['NewRetail'] < $discount['NewRetail']) {
                    $discount = $disc;
                }
            }

            // ako imamo rabat onda cijena dolazi odande, inače uzimamo veleprodajnu cijenu artikla
            $productPrice = $rebate['CalculatedNetto'] ?? $result['PriceWholesale'];

            // ako imamo još i akciju koja je povoljnija od rabata.
            if ($discount && $discount['NewWholesale'] <= $productPrice) {
                $productPrice = $discount['OriginalWholesale'];
                $regularPrice = $productPrice;
                $salePrice = $discount['NewWholesale'];
                $netPrice = $discount['OriginalWholesale'];
                $rebatePercentage = $discount['Rate'];
            } else {
                $regularPrice = '';
                $salePrice = '';
                $netPrice = $rebate['PriceWholesale'] ?? $result['PriceWholesale'];
                $rebatePercentage = $rebate['Rate'] ?? 0;
            }

            $this->pricesById[$id] = [
                '_net_price' => $netPrice,
                '_price' => $productPrice,
                '_rebate_percentage' => $rebatePercentage,
                '_regular_price' => $regularPrice,
                '_sale_price' => $salePrice,
            ];
        }

        return $this->pricesById[$id];
    }

    protected function load(): void
    {
        if (count($this->queuedIds) === 0) {
            return;
        }

        $skus = [];
        $tireIds = [];

        foreach ($this->queuedIds as $preloadId) {
            if (isset($this->resultsById[$preloadId]) || in_array($preloadId, $this->ignoredIds)) {
                continue;
            }

            if ($preloadId > Products::ID_OFFSET_GROUPED && $preloadId < Products::ID_OFFSET_TIRES) {
                $this->ignoredIds[] = $preloadId;
                continue;
            }

            if ($preloadId > Products::ID_OFFSET_TIRES) {
                $tireIds[$preloadId] = (int)($preloadId - Products::ID_OFFSET_TIRES);
                continue;
            }

            $skus[$preloadId] = get_post_meta($preloadId, '_sku', true);
        }

        if (count($tireIds) > 0 || count($skus) > 0) {
            try {
                $data = $this->gath->getPricesBySkus(array_values($skus), array_values($tireIds));
            } catch (\Exception $e) {
                $this->logger->crit(sprintf(
                    "Došlo je do greške prilikom dohvaćanja cijena: %1\$s\n%3\$s\nŠifre: %2\$s\n%3\$s",
                    $e->getMessage(),
                    implode(', ', $skus),
                    str_repeat('-', 80)
                ));

                $data = ['Results' => []];
            }

            foreach ($data['Results'] as $result) {
                $productId = array_search($result['ProductCode'], $skus);

                if (!$productId)
                    $productId = array_search($result['ProductId'], $tireIds);

                $this->resultsById[$productId] = $result;
            }
        }

        $this->queuedIds = [];
    }
}